@@ -118,7 +118,7 @@ use constant IS_CGI      => IS_MOD_PERL || exists $ENV{'GATEWAY_INTERFACE'};
 # we use a Hash to store the configuration. This is simpler to pass to functions.
 our %config;
 
-$config{version}                  = "23.02.15";  # year.month.day
+$config{version}                  = "24.01.19";  # year.month.day
 
 my $dirname = dirname(__FILE__);
 
@@ -673,6 +673,10 @@ if (not $error) {
     . " -usb -device usb-tablet"
     . " -vga $session{video}";
     
+  # add copy/paste support (not always functional)
+  $cmd .= " -chardev qemu-vdagent,id=ch1,name=vdagent,clipboard=on -device virtio-serial-pci"
+  . " -device virtserialport,chardev=ch1,id=ch1,name=com.redhat.spice.0";
+    
 
   # performance options: network 
   #   see: https://elinux.org/images/3/3b/Kvm-network-performance.pdf
@@ -852,7 +856,7 @@ if (not $error) {
   if (not $session{oneshot}) {
     $output_share = "<button onClick=\"SelfCopy(this.id)\" id=\"$session{url} $session{token_vnc}\" style=\"padding:0;border:0;background:none;outline:none\"><img alt=\"copy\" src=\"$config{http}://$session{server_name}/$config{service}/images/share-button.png\" title=\"Copy link to clipboard, share it with your colleagues.\" height=40></button>";
     $output_persistent = "<li>You can close the browser and reconnect any time (within life-time) with the link above.</li>" .
-    "<li>Select the <span style=\"color:green\">[Manage sessions]</span> item in the service login page to list, reconnect or abort your sessions.</li>" .
+    "<li>Select the <b><span style=\"color:green\">[Manage sessions]</span></b> item in the service login page to list, reconnect or abort your sessions.</li>" .
     "<li>You can collaborate in the same session with your colleagues. Just send them the link above.</li>" .
     "<li>Please <b>shut-down the machine properly</b> (do not just logout or suspend).</li>";
   }
@@ -892,18 +896,28 @@ if (not $error) {
     <h1>Hello $session{user} !</h1>
     
     <p>Your machine $config{service} $session{machine} has just started. 
-    Click on the following link.</p>
     
     <div style="text-align: center;">
       <a href=$session{url} target=_blank>$icon</a>
-      <br></br>
-      <b><a href="$session{url}" target=_blank>
-      <img alt="connect" 
+      <br><br>
+      
+      <table style="width:100%;"><tr>
+      
+      <td><b><a href="$session{url}" target=_blank>
+      <img alt="connect"
         src="$config{http}://$session{server_name}/$config{service}/images/connect-button.jpeg" 
-        title="Connect !" height=40></a></b> 
-        $output_share
+        title="Connect !" height=40></a></b>
+      $output_share</td>
+      
+      <td style="text-align: right"><form action="$config{http}://$session{server_name}/cgi-bin/qemu-web-desktop.pl" method=post>
+      <input type=hidden name=session_stop value="$session{name}">
+      <input type=image alt="stop" src="$config{http}://$session{server_name}/$config{service}/images/stop-button.png" title=STOP height="40">
+      </form></td>
+      
+      </tr></table>
         $output_token
     </div>
+
     
     <p>
     NOTES: <ul>
@@ -911,10 +925,6 @@ if (not $error) {
     <li>The virtual machine is created on request, and not kept. 
       Your work <b>must be saved elsewhere</b> 
       (e.g. mounted disk, ssh/sftp, Dropbox, OwnCloud...).</li>
-    <li>To <b>kill</b> this session, click on <form action="$config{http}://$session{server_name}/cgi-bin/qemu-web-desktop.pl" method=post target=_blank>
-    <input type=hidden name=session_stop value="$session{name}">
-    <input type=image alt="stop" src="$config{http}://$session{server_name}/$config{service}/images/stop-button.png" title=STOP height="32">
-    </form></li>
     </ul></p>
 
     <hr>
@@ -1199,9 +1209,8 @@ sub session_run_script {
     $content=~s/[^\x00-\x7f]//g;
     if (length($content) < $config{service_max_script_length}) {
     
-      # test if script is active
+      # replace symbols. Script is known to be active from previous call to session_test_script
       ($TF, $content) = session_test_script(\%session, $content);
-      if (not $TF) { return ""; }
       
       # store script in our 'snapshot' directory for local exec.
       my ($script_handle, $script_name) = tempfile(TEMPLATE => "$session{dir_snapshot}/configuration_script_XXXXXXXX");
@@ -1249,7 +1258,7 @@ sub service_housekeeping {
   my $now         = localtime();
   foreach my $snapshot (glob("$dir/$service"."_*")) {
     
-    if (-d $snapshot) { # is a snapshot directory
+    if (-d $snapshot && time - (stat($snapshot))[9] > $config{boot_delay}) { # is a snapshot directory
       my $snaphot_name = fileparse($snapshot); # just the session name
       
       if (not -e "$cfg/$snaphot_name.json") {
@@ -1277,7 +1286,7 @@ sub service_housekeeping {
     
     # session exists, load JSON
     my $session_ref = session_load(\%config, $json);
-    if ($session_ref) {
+    if ($session_ref && time - (stat($json))[9] > $config{boot_delay}) {
       my %session = %{ $session_ref };
       if (not -d $session{dir_snapshot}) { 
         print STDERR $0." [$now] housekeeping: $json (left-over, no snapshot)\n";
